--################################################################
-- written by CrisOHara
-- Version 2
-- 6 January 2023
-- This software is free
--
-- The script is developed for x9lite-S openTx 2.3.11 (but likely to run on some other radio)
-- Cause some of the used values depend on "magic" numbers it may be nessesary to adapt for other radios
-- The hardware dependency is base on the fact that different type of radios do have a different amount of switches.
-- Also different software version may have an effect.
-- So please check on companion first (or at least use a copy of an existing model)
--
-- Script can be used as telemetry script in a telemetry view (but has to run only once so it should be removed after first run)
--   or with execute when selected from the file menu direct from sdcard (that's what I prefer)
-- Be aware that you have to select to model as active model first.
--
-- Hints:
--
--
-- this example script creates: Mixxers to mix Trainer source an stick source 
--
--   CH25  = input student aileron
--   CH26  = input student elevator
--   CH27  = input student throttle
--   CH28  = input student ruder
--   CH29  = Aileron
--   CH30  = Elevator
--   CH31  = Trottle
--   Ch32  = Ruder
--
-- no warranty: script is to be used on your own risk
--
--######################################################################

--############################ looking for Mixer ######################
local function my_lookForMixer (channelNr, sourceNr)

	local result = -1;
	lineAmount = model.getMixesCount(channelNr);
	if lineAmount == 0 then return lineAmount, result; end	
	for i=0, lineAmount-1 do
	    mixerData = model.getMix( channelNr, i );
	    if mixerData.source == sourceNr then result = 1; end
	  end
	return lineAmount, result;
end


-- ############################# init function ##################
-- global variables
local errorOnIndex = 0;
local indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML
-- function
local function Chadef_init()

local lowWeight = 60;
local highWeight = 80;

--##### defining of Indes for LS01, TR1, CH01, Max ### depends on radio Type ### x9Lite = 36

	chunk, err = loadScript("/SCRIPTS/FUNCTIONS/trainer/indLib.lua");
	if err then
	   errorOnIndex = 1;
	   return;
	   end;
        local script=chunk();
        indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML = script.init();



--#####  create mixer for Trainer (Ch29, CH30, CH31, CH32) ########################

local mixerData = { name, source, weight, offset, switch, multiplex, curveType, curveValue, flightModes, carryTrim, mixWarn, delayUp, delayDown, speedUp, speedDown}
local sourceNr = 0;

-- ######  calibrating students inputs with weight, offset and channelorder ###################

-- creating Mixer Ch25  which does calibrating of aileron  (Tr1 for Multiplex)
	channelNr = 25;
	sourceNr  = indexSourceTR01;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="Tr-ail", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

-- creating Mixer Ch26  which does calibrating of elevator (Tr2 for Multiplex)
	channelNr = 26;
	sourceNr  = indexSourceTR01 + 1;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="Tr-ele", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

-- creating Mixer Ch27  which does calibrating of throttle (Tr4 for Multiplex)
	channelNr = 27;
	sourceNr  = indexSourceTR01 + 3;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="Tr-thr", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

-- creating Mixer Ch28  which does calibrating of ruder    (Tr3 for Multiplex)

	channelNr = 28;
	sourceNr  = indexSourceTR01 + 2;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="Tr-rud", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end


-- ######  mixing master with student radio ###################

-- creating Mixer Ch29 (Aileron)
	channelNr = 29;

	sourceNr  = indexSourceRud + 3;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="M-Ail", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

	sourceNr  = indexSourceCH01 + 24;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="T-Ail1", source=sourceNr,  weight=lowWeight, offset=0, switch=(indexSwitchLS01+14), multiplex=2, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    mixerData = { name="T-Ail2", source=sourceNr,  weight=highWeight, offset=0, switch=(indexSwitchLS01+15), multiplex=2, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount+1, mixerData);
	    end


-- creating Mixer Ch30 (Elevator)
	channelNr = 30;

	sourceNr  = indexSourceRud + 1;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="M-Ele", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

	sourceNr  = indexSourceCH01 + 25
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="T-Ele1", source=sourceNr,  weight=lowWeight, offset=0, switch=(indexSwitchLS01+14), multiplex=2, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    mixerData = { name="T-Ele2", source=sourceNr,  weight=highWeight, offset=0, switch=(indexSwitchLS01+15), multiplex=2, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount+1, mixerData);
	    end

-- creating Mixer Ch31 (Trottle)
	channelNr = 31;

	sourceNr  = indexSourceRud + 2;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="M-Thr", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

	sourceNr  = indexSourceCH01 + 26;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="T-Thr", source=sourceNr,  weight=100, offset=0, switch=(indexSwitchLS01+13), multiplex=2, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
-- uncomment to give throttle to student	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

-- creating Mixer Ch32 (Ruder)
	channelNr = 32;

	sourceNr  = indexSourceRud;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="M-Rud", source=sourceNr,  weight=100, offset=0, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    end

	sourceNr  = indexSourceCH01 + 27;
	lineAmount, status = my_lookForMixer (channelNr-1, sourceNr);
	if status == -1 then
	    mixerData = { name="T-Rud1", source=sourceNr,  weight=lowWeight, offset=0, switch=(indexSwitchLS01+14), multiplex=2, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount, mixerData);
	    mixerData = { name="T-Rud1", source=sourceNr,  weight=highWeight, offset=0, switch=(indexSwitchLS01+15), multiplex=2, curveType=0, curveValue=0, flightModes=512,
                        carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	    model.insertMix(channelNr-1, lineAmount+1, mixerData);
	    end

end


-- ############################### run function #####################
local function Chadef_run(event)
	lcd.clear();

	if lineSizeMID == nil then lineSizeMID = 17; lineSizeSML = 9; end
	lcd.drawText (0, 0,"Trainer Mixers", MIDSIZE);
	lcd.drawText (0, lineSizeMID,"by swap", MIDSIZE);
	lcd.drawText (0, 2*lineSizeMID,"no throttle", MIDSIZE);
	lcd.drawText (0, 3*lineSizeMID,"## Created ##", MIDSIZE+INVERS);
	return 0;
end

return { init=Chadef_init, run=Chadef_run };