--################################################################
-- written by CrisOHara
-- Version 1
-- 2 January 2023
-- This software is free
--
-- The script is developed for x9lite-S openTx 2.3.15 (but likely to run on some other radio)
-- Cause some of the used values depend on "magic" numbers it may be nessesary to adapt for other radios
-- The hardware dependency is base on the fact that different type of radios do have a different amount of switches.
-- Also different software version may have an effect.
-- So please check on companion first (or at least use a copy of an existing model)
--
-- Script can be used as telemetry script in a telemetry view (but has to run only once so it should be removed after first run)
--   or with execute when selected from the file menu direct from sdcard (that's what I prefer)
-- Be aware that you have to select to model as active model first.
--
-- But anyway this script can be used as an example/template how to do your standard setup for logical switches and special functions.
--
-- Hints:
-- Many of the numbers used in the functions model.setLogicalSwitch or model.setCustomFunction (like the switch index) are difficult to find out.
-- One possible way to find out is: 
--       Define the logical switch or special function manually in your radio. 
--       Then use the model.getLogicalSwitch or model.getCustomFunction and read out the values.
--  If you need a switch like "|L01 (NOT L01)" try a negativ value (for xLite L02=37 ; |L02=-37)
-- To find the number of the hardware switch or functions you can also count through the scroll menu when you select the 
--   hardwareswitch/function on the radio. (does work for hardwareswitches if they are the first in the scroll list  
--   and does work for "func")
--
-- this script:
--    reads the models Input lines and modifies redirects the source to ch29-ch32
--    (standard) input-lines for trainer sources (Tr1, Tr2, Tr4, Tr3) (used Multiplex channel order)
--    due to an error in a standard LUA function all created/modified input-lines 
--           have to be corrected manually (Trim has to be defined cause it's missing))
-- This is the standard I use for all my models.
--
-- no warranty: script is to be used on your own risk
--
--######################################################################


--############################# init function ##################
-- global variables
local errorOnIndex = 0;
local  indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML
-- function
local function InpBud_init()

--##### defining of Indes for LS01, TR1, CH01, Max ### depends on radio Type ### x9Lite = 36

	chunk, err = loadScript("/SCRIPTS/FUNCTIONS/trainer/indLib.lua");
	if err then
	   errorOnIndex = 1;
	   return;
	   end;
        local script=chunk();
        indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML = script.init();


--#####  create inputs for Trainer ########################

local inputData = { name, inputName, source, weight, offset, switch, curveType, curveValue, carryTrim, flightModes}
local newData = { name, inputName, source, weight, offset, switch, curveType, curveValue, trimSource, flightModes}
local inputNumber = 0;
local originalLineAmount = 0;

	for inputNumber = 0, 31 do
	
	  originLineAmount = model.getInputsCount(inputNumber);
	  if (originLineAmount > 0) then

	     for inputLineNr = 0, (originLineAmount-1) do
		inputData = model.getInput(inputNumber, inputLineNr);
		  newData.name = inputData.name
		  newData.inputName = inputData.inputName;
		  newData.weight = inputData.weight;
		  newData.offset = inputData.offset;
		  newData.switch = inputData.switch;
		  newData.curveType = inputData.curveType;
		  newData.curveValue = inputData.curveValue;
		  newData.flightModes = inputData.flightModes;
			if (inputData.source == indexSourceRud + 3) then           --###### Aileron
			      newData.source = indexSourceCH01 + 28;
			      newData.trimSource = 4;
			      if (newData.curveType == 0 and newData.curveValue == 0)  then
				 newData.curveType = 1;
				 newData.curveValue = 50;
				 end;
			      model.insertInput(inputNumber, inputLineNr, newData);
			      model.deleteInput(inputNumber, (inputLineNr + 1));
			   end
			if (inputData.source == indexSourceRud + 1) then           --###### Elevator
			      newData.source = indexSourceCH01 + 29;
			      newData.trimSource = 2;
			      if (newData.curveType == 0 and newData.curveValue == 0)  then
				 newData.curveType = 1;
				 newData.curveValue = 50;
				 end;
			      model.insertInput(inputNumber, inputLineNr, newData);
			      model.deleteInput(inputNumber, (inputLineNr + 1));
			   end
			if (inputData.source == indexSourceRud + 2) then           --###### Trottle
			      newData.source = indexSourceCH01 + 30;
			      newData.trimSource = 3;
			      model.insertInput(inputNumber, inputLineNr, newData);
			      model.deleteInput(inputNumber, (inputLineNr + 1));
			   end
			if (inputData.source == indexSourceRud) then           --###### Ruder
			      newData.source = indexSourceCH01 + 31;
			      newData.trimSource = 1;
			      if (newData.curveType == 0 and newData.curveValue == 0)  then
				 newData.curveType = 1;
				 newData.curveValue = 50;
				 end;
			      model.insertInput(inputNumber, inputLineNr, newData);
			      model.deleteInput(inputNumber, (inputLineNr + 1));
			   end
	    end
	  end
	end

end



-- ############################### run function #####################
local function InpBud_run(event)
	lcd.clear();

	if lineSizeMID == nil then lineSizeMID = 17; lineSizeSML = 9; end
	lcd.drawText (0, 0,"Inputs", MIDSIZE);
	lcd.drawText (0, lineSizeMID,"for Trainer", MIDSIZE);
	lcd.drawText (0, 2*lineSizeMID,"or Spotter", MIDSIZE);
	if errorOnIndex == 1 then 
		lcd.drawText (0, 3*linesizeMID,"file indLib not found", MIDSIZE);
	      else
		lcd.drawText (0, 3*lineSizeMID,"## Created ##", MIDSIZE+INVERS);
	      end;	
	return 0;
end

return { init=InpBud_init, run=InpBud_run };