--################################################################
-- written by CrisOHara
-- Version 2
-- 3 January 2022
-- This software is free
--
-- The script is developed for x9lite-S openTx 2.3.11 (but likely to run on some other radio)
-- Cause some of the used values depend on "magic" numbers it may be nessesary to adapt for other radios
-- The hardware dependency is base on the fact that different type of radios do have a different amount of switches.
-- Also different software version may have an effect.
-- So please check on companion first (or at least use a copy of an existing model)
--
-- Script can be used as telemetry script in a telemetry view (but has to run only once so it should be removed after first run)
--   or with execute when selected from the file menu direct from sdcard (that's what I prefer)
-- Be aware that you have to select to model as active model first.
-- Some definitions can only be created if specific sensor values are available.
--
-- But anyway this script can be used as an example/template how to do your standard setup for logical switches and special functions.
--
-- Hints:
-- Many of the numbers used in the functions model.setLogicalSwitch or model.setCustomFunction (like the switch index) are difficult to find out.
-- One possible way to find out is: 
--       Define the logical switch or special function manually in your radio. 
--       Then use the model.getLogicalSwitch or model.getCustomFunction and read out the values.
--  If you need a switch like "|L01 (NOT L01)" try a negativ value (for xLite L02=37 ; |L02=-37)
-- To find the number of the hardware switch or functions you can also count through the scroll menu when you select the 
--   hardwareswitch/function on the radio. (does work for hardwareswitches if they are the first in the scroll list  
--   and does work for "func")
--
-- this example script creates:
--    (standard) logical switches  1 to 7   and  61 to 64
--    (standard) spezial functions 1 to 9
-- This is the standard I use for all my models.
--
-- no warranty: script is to be used on your own risk
--
--######################################################################


--############################ looking for sensor ######################

local function my_lookForSens(sensorName)

	for i=0, 40 do
	    k_sensor = model.getSensor(i);
	    if (k_sensor ~= nil) and (k_sensor.name == sensorName) then return i ; end
	end
     return -1;
end


-- ############################# init function ##############################################################
-- global variables
local errorOnIndex = 0;
local 	indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML
-- function 

local function basDef_init()

local valueName = " ";


--##### defining of Indes for LS01, TR1, CH01, Max ### depends on radio Type ### x9Lite = 36

	chunk, err = loadScript("/SCRIPTS/FUNCTIONS/trainer/indLib.lua");
	if err then
	   errorOnIndex = 1;
	   return;
	   end;
        local script=chunk();
        indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML = script.init();


--#### L01 L02 -> SF01 SF02
--#### check engine voltage ################################################
local	valueName2="MAX"
        valueID2= getFieldInfo(valueName2) and getFieldInfo(ValueName2).id or nil
	valueName = "VFAS";
       	sensListID = my_lookForSens(valueName);
	if sensListID >= 0 then
	    valueID = getFieldInfo(valueName) and getFieldInfo(valueName).id or nil

	    model.setLogicalSwitch (0, {func=4, v1=valueID, v2=110, delay=100, ["and"]=-(indexSwitchLS01+1)});
	     model.setCustomFunction(0, {switch=(indexSwitchLS01 + 0), func=6, value=indexSourceMAX, active=1 });
	     model.setCustomFunction(1, {switch=(indexSwitchLS01 + 0), func=12, value=valueID, active=10 });
	    model.setLogicalSwitch (1, {func=4, v1=valueID, v2=104, delay=50 });
	     model.setCustomFunction(2, {switch=(indexSwitchLS01 + 1), func=6, value=indexSourceMAX, active=1 });
	     model.setCustomFunction(3, {switch=(indexSwitchLS01 + 1), func=12, value=valueID, active=5 });
	 else
	    valueName = "A2";
            sensListID = my_lookForSens(valueName);
	    if sensListID >= 0 then
	      	valueID = getFieldInfo(valueName) and getFieldInfo(valueName).id or nil
		model.setLogicalSwitch (0, {func=4, v1=valueID, v2=110, delay=100, ["and"]=-(indexSwitchLS01+1) });
		 model.setCustomFunction(0, {switch=(indexSwitchLS01 + 0), func=6, value=indexSourceMAX, active=10 });
		 model.setCustomFunction(1, {switch=(indexSwitchLS01 + 0), func=12, value=valueID, active=10 });
	 	model.setLogicalSwitch (1, {func=4, v1=valueID, v2=104, delay=50 });
	     	 model.setCustomFunction(2, {switch=(indexSwitchLS01 + 1), func=6, value=indexSourceMAX, active=5 });
	     	 model.setCustomFunction(3, {switch=(indexSwitchLS01 + 1), func=12, value=valueID, active=5 });
             end
	 end

--#### L03 -> SF03
--#### check receiver voltage ##################################################
	valueName = "RxBt";
	sensListID = my_lookForSens(valueName);
	if sensListID >= 0 then
	    valueID = getFieldInfo(valueName) and getFieldInfo(valueName).id or nil
	    model.setLogicalSwitch (2, {func=4, v1=valueID, v2=48, delay=1 });
	     model.setCustomFunction(4, {switch=(indexSwitchLS01 + 2), func=6, value=indexSourceMAX, active=1 });
	     model.setCustomFunction(5, {switch=(indexSwitchLS01 + 2), func=12, value=valueID, active=5 });
	 end

--#### L04 -> SF04 SF05
--#### reset timer #############################################################
	model.setLogicalSwitch (3, {func=11, v1=12, v2=(-129), v3=-1});
	 model.setCustomFunction(6, {switch=(indexSwitchLS01 + 3), func=3, value=0, active=1 });
	 model.setCustomFunction(7, {switch=(indexSwitchLS01 + 3), func=3, value=1, active=1 });

--#### L05 SF06
--#### reset telemetry ALT and VSpd three seconds after receiver GR6 switch on #### L05 ##################
	valueName = "Alt";
        sensListID = my_lookForSens(valueName);
	if sensListID >= 0 then
	    model.setLogicalSwitch (4, {func=11, v1=indexSwitchTele, v2=(-129+24), v3=-1});
	    model.setCustomFunction(8, {switch=(indexSwitchLS01 + 4), func=3, value=(sensListID + 5), active=1 });
          end
	valueName = "VSpd";
        sensListID = my_lookForSens(valueName);
	if sensListID >= 0 then
	    model.setLogicalSwitch (4, {func=11, v1=indexSwitchTele, v2=(-129+24), v3=-1});
	    model.setCustomFunction(9, {switch=(indexSwitchLS01 + 4), func=3, value=(sensListID + 5), active=1 });
          end


--#### L06 L07 
--#### Save engine cut off #### L06 L07 and CF11   #####################################################
	valueID = 3; -- getFieldInfo("thr") and getFieldInfo("thr").id or nil
	model.setLogicalSwitch (5, {func=4, v1=valueID, v2=-99, ["and"]=12});
	model.setLogicalSwitch (6, {func=18, v1=(indexSwitchLS01 + 5), v2=10, delay=1});
 	model.setCustomFunction(10, {switch=-(indexSwitchLS01 + 6), func=0, value=-100, param=2, active=1 });


--#### L61
--#### res_T Res training script
	  model.setLogicalSwitch (60, {func=9, v1=15, v2=15, v3=3});


--#### L62 L63 l64
--#### TelSxx script (telemetry announcement) ##############################################
	  model.setLogicalSwitch (61, {func=11, v1=18, v2=(-129), v3=3, duration=0 });
	  model.setLogicalSwitch (62, {func=11, v1=18, v2=(-126), v3=-1, duration=0 });
	  model.setLogicalSwitch (63, {func=9, v1=21, v2=21 });

--#### Timer 1 und 2
--#### Timer werden nur angelegt nur wenn sie noch nicht vorhanden sind ##########################
local	timerData = model.getTimer(0);
        if (timerData.mode == 0) then 
		model.setTimer(0,{mode=3, start=180, value=180, countdownBeep=2, minuteBeep=true, persistent=0, name="Rel"});
		end;
	timerData = model.getTimer(1);
        if (timerData.mode == 0) then
		model.setTimer(1,{mode=16,  start=240, value=240, countdownBeep=2, minuteBeep=true, persistent=0, name="Abs"});
		end;
 
end




-- ############################### run function #####################
local function basDef_run(event)
	lcd.clear();

	if lineSizeMID == nil then lineSizeMID = 17; lineSizeSML = 9; end
	lcd.drawText (0, 0,"Log-Switches", MIDSIZE);
	lcd.drawText (0, lineSizeMID,"Spec-Functions", MIDSIZE);
	lcd.drawText (0, 2*lineSizeMID,"Timer", MIDSIZE);
	if errorOnIndex == 1 then 
		lcd.drawText (0, 3*lineSizeMID,"file indLib not found", MIDSIZE);
	     else
		lcd.drawText (0, 3*lineSizeMID,"## Created ##", MIDSIZE+INVERS);
	     end
	return 0;
end

return { init=basDef_init, run=basDef_run };