--#############################################################
--
-- written by ChrisOHara 18.01.2023 
-- this softare is free, no garantee so useage on own risk
--
-- the script is used to "calibrate" the ppm-input from a students radio for one stick
-- it calculates and stores the values (weight, offset, tr-source) in the specified channel.
-- so also a kind of channel mapping is done.
--
--##############################################################

-- definitions for different sticks (aileron, elevator, ruder)

local stickName = "ELEVATOR";
local mixerName = "Tr-ele";
local positiveDirection = "TOP";
local negativeDirection = "BOTTON";
local channelNr = 26;


--############ LUA script #######################################

local waitTime =600;
local step = 0;
local minimum = {};
local maximum = {};
local TRxxFieldID = {};
local myWeight;
local myOffset;
local indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML;

-- ############################# init function ##################
local function calStudentSticks_init()

    for ii = 0, 3 do
        TRxxFieldID[ii] = getFieldInfo("trn"..(ii+1)) and getFieldInfo("trn"..(ii+1)).id or nil;
      end

    -- defining of Indes for LS01, TR1, CH01, Max ### depends on radio Type ### x9Lite = 36
    chunk, err = loadScript("/SCRIPTS/FUNCTIONS/trainer/indLib.lua");
    if err then
        errorOnIndex = 1;
        return;
      end;
    local script=chunk();
    indexSwitchLS01, indexSourceTR01, indexSourceMAX, indexSourceCH01, indexSwitchTele, indexSourceRud, lineSizeMID, lineSizeSML = script.init();

end


--############################# input screen ########################
local function looking_for_extrems()
local value

     for ii=0 , 3 do
         value = getValue(TRxxFieldID[ii]);
         if (value < minimum[ii]) then minimum[ii] = value; end
         if (value > maximum[ii]) then maximum[ii] = value; end
       end;
end

--########################### checking all sticks near center #######
local function checkAllSticks_func()

local returnValue = 0;
      for ii = 0, 3 do
          if math.abs(getValue(TRxxFieldID[ii])) > 200 then
              returnValue = 1;
            end;
        end;
      return returnValue;
end

-- ############################### run function #####################
local function  minMax_func()

local trainerStickNr = 0;
local maxValue = 0;

     for ii = 0, 3 do
         if math.abs(minimum[ii]) >= math.abs(maxValue) then
              maxValue = minimum[ii];
              trainerStickNr = ii;
	      end
         if math.abs(maximum[ii]) >= math.abs(maxValue) then
              maxValue = maximum[ii];
              trainerStickNr = ii;
	      end
       end

     return trainerStickNr, maxValue;

end


-- ############################### run function #####################
local function calStudentSticks_run(event)

     lcd.clear()

     if (step == 0) then 
         if (checkAllSticks_func() == 0) then
             endTime = getTime() + waitTime;
             minimum[0]=0; minimum[1]=0; minimum[2]=0; minimum[3]=0;
             maximum[0]=0; maximum[1]=0; maximum[2]=0; maximum[3]=0;
             step = 1;
	   else
             lcd.drawText (0,  lineSizeMID,"move all sticks ",MIDSIZE);
             lcd.drawText (0,2*lineSizeMID,"near centerpoint",MIDSIZE);
           end
      end;

     if (step == 1) and (getTime() <= endTime)  then 
	  lcd.drawText (0,0,stickName,MIDSIZE);
	  lcd.drawText (0,lineSizeMID,"move stick to",MIDSIZE);
          lcd.drawText (0,2*lineSizeMID,"maximum "..positiveDirection,MIDSIZE);
          lcd.drawText (0,3*lineSizeMID,"and back to center",MIDSIZE);
          looking_for_extrems();
         end;
     if (step == 1) and (getTime() > endTime)  then 
           trainerStickNrOne, maxValueOne = minMax_func();
           endTime = getTime() + waitTime;
	   minimum[0]=0; minimum[1]=0; minimum[2]=0; minimum[3]=0;
	   maximum[0]=0; maximum[1]=0; maximum[2]=0; maximum[3]=0;
           step = 2;
         end;

     if (step == 2) and (getTime() <= endTime)  then 
	  lcd.drawText (0,0,stickName,MIDSIZE);
	  lcd.drawText (0,lineSizeMID,"move stick to",MIDSIZE);
          lcd.drawText (0,2*lineSizeMID,"maximum "..negativeDirection,MIDSIZE);
          lcd.drawText (0,3*lineSizeMID,"and back to center",MIDSIZE);
          looking_for_extrems();
         end
     if (step == 2) and (getTime() > endTime)  then
           trainerStickNrTwo, maxValueTwo = minMax_func();
           myWeight = math.floor(204800/(maxValueOne - maxValueTwo));
	   if trainerStickNrOne ~= trainerStickNrTwo then
                 lcd.drawText (0,35,"failed - different sticks",MIDSIZE);
                 return 0;
              end;
           endTime = getTime() + waitTime;
           step = 3;
         end

      if ((step == 3) and (getTime() <= endTime)) then
	   lcd.drawText (0,0,stickName,MIDSIZE);
	   lcd.drawText (0,lineSizeMID,"move stick to",MIDSIZE);
           lcd.drawText (0,2*lineSizeMID,"CENTER",MIDSIZE);
         end

      if ((step == 3) and (getTime() > endTime)) then
          myOffset = math.floor( ((-getValue(TRxxFieldID[trainerStickNrOne])*myWeight) / 1024) );

          -- creating Mixer
	    local mixerData = { name=mixerName, source=(indexSourceTR01 + trainerStickNrOne) ,  weight=myWeight, offset=myOffset, multiplex=0, curveType=0, curveValue=0, flightModes=512,
                             carryTrim=0, mixWarn=0, delayUp=0, delayDown=0, speedUp=0, speedDown=0}
	     model.deleteMix(channelNr-1, 0);
	     model.insertMix(channelNr-1, 0, mixerData);
	  step = 4;
	end;

      if (step == 4) then
          local lineNr = 0;      
          lcd.drawText (0,lineNr, stickName.." - ch"..channelNr,MIDSIZE);                    lineNr = lineNr + lineSizeMID;   
          lcd.drawText (0,lineNr,"TRxx-Nr  "..trainerStickNrOne+1,MIDSIZE);                  lineNr = lineNr + lineSizeMID; 
          lcd.drawText (0,lineNr,"weight "..myWeight,SMLSIZE);                               lineNr = lineNr + lineSizeSML; 
          lcd.drawText (0,lineNr,"offset "..myOffset,SMLSIZE);                               lineNr = lineNr + lineSizeSML; 
          lcd.drawText (0,lineNr,"maxOne "..maxValueOne.."  maxTwo "..maxValueTwo,SMLSIZE)
        end

     return 0

end


return { init=calStudentSticks_init, run=calStudentSticks_run}